
from torchvision.models import resnet18, resnet50
import torch
import torchvision.datasets as dset
import torchvision.transforms as transforms




def VNRF_rmpercent(model, percent, return_model=False):
    """
    model: "ResNet18"
    delete_layer: ['conv1.weight', 'bn1.weight', 'bn1.bias', 'bn1.running_mean', 'bn1.running_var', 'bn1.num_batches_tracked']
    """
    state_dict = model.state_dict()
    keys  = state_dict.keys()
    
    l1_norm = {}
    print(keys)
    for key in keys:
        if "conv" in key:

            weight = state_dict[key].data
            res = torch.linalg.norm(weight.clone().reshape(weight.size(0), -1), ord=1, dim=-1)
            mean_l1 = torch.mean(torch.tensor(res))
            l1_norm[key] = {"l1_norm":res, "mean":mean_l1}

    for layer in keys:

        if "conv" in layer:


            layer_l1_norm = l1_norm[layer]["l1_norm"]
            sorted_index = torch.argsort(layer_l1_norm, descending=True)
            num_filters = int(state_dict[layer].data.size(0)*(percent*0.01))

            for i in range(num_filters):

                tem = state_dict[layer].data[sorted_index[i], :, :, :]
                state_dict[layer].data[sorted_index[i], :, :, :] = torch.zeros(tem.size())

    if return_model:
        model.load_state_dict(state_dict)
        return model
    return state_dict


def VNRF_generate(model, percent):
    """
    model: "ResNet18"
    delete_layer: ['conv1.weight', 'bn1.weight', 'bn1.bias', 'bn1.running_mean', 'bn1.running_var', 'bn1.num_batches_tracked']
    """
    state_dict = model.state_dict()
    keys  = state_dict.keys()
    
    l1_norm = {}

    for key in keys:
        if "conv" in key:

            weight = state_dict[key].data
            res = torch.linalg.norm(weight.clone().reshape(weight.size(0), -1), ord=1, dim=-1)
            mean_l1 = torch.mean(torch.tensor(res))
            l1_norm[key] = {"l1_norm":res, "mean":mean_l1}

  
    for layer in keys:

        if ("conv" in layer) and (("layer4" in layer) or ("layer3") in layer):

            layer_l1_norm = l1_norm[layer]["l1_norm"]
            sorted_index = torch.argsort(layer_l1_norm, descending=True)
            num_filters = int(state_dict[layer].data.size(0)*(percent*0.01))
            for i in range(num_filters):
                tem = state_dict[layer].data[sorted_index[i], :, :, :]
                state_dict[layer].data[sorted_index[i], :, :, :] = torch.zeros(tem.size())


    return state_dict


if __name__ == "__main__":

    device = 'cuda' if torch.cuda.is_available() else 'cpu'
    net =ResNet18()
    net = torch.nn.DataParallel(net)

    state_dict = torch.load('./weights/cifar10_resnet18_id_cifar_333.pth.tar')
    net.load_state_dict(state_dict)

    # state_dict = VNRF(net,['conv1.weight'], 0)
    # net.load_state_dict(state_dict)
    net.to(device)
    # print(net.state_dict()['conv1.weight'])

    transform_test = transforms.Compose([
        transforms.ToTensor(),
        transforms.Normalize((0.4914, 0.4822, 0.4465), (0.2023, 0.1994, 0.2010)),
    ])
    test_data = dset.CIFAR10("/path/to/cifar", train=False, transform=transform_test, download=True)
    testloader = torch.utils.data.DataLoader(test_data,
                                                    batch_size=512,
                                                    shuffle=False,
                                                    num_workers=4,
                                                    pin_memory=True)
    
    total=0
    correct=0

    with torch.no_grad():
        for batch_idx, (inputs, targets) in enumerate(testloader):
            net.eval()
            inputs, targets = inputs.to(device), targets.to(device)
            outputs = net(inputs)
            # loss = criterion(outputs, targets)
            # test_loss += loss.item()
            _, predicted = outputs.max(1)
            total += targets.size(0)
            correct += predicted.eq(targets).sum().item()

            # progress_bar(batch_idx, len(testloader), 'Acc: %.3f%% (%d/%d)'
            #              % ( 100.*correct/total, correct, total))
        print('Acc: %.3f%% (%d/%d)' % ( 100.*correct/total, correct, total))

